<?php

/**
 * @file
 * Administrative configuration for the Project Documentation module.
 */

/**
 * Callback to generate projectdocumentation nodes.
 */
function projectdocumentation_generate($form, &$form_state) {
  $projects = array();
  $result = db_query("SELECT filename, name, info, bootstrap, schema_version, status FROM {system} WHERE type = 'module'");
  while ($module = $result->fetchObject()) {
    // Exclude non-contributed modules (path does not start with 's').
    if ('s' != $module->filename[0] ) {
      continue;
    }
    $info = unserialize($module->info);
    $shortname = $module->name;
    if (empty($info['project'])) {
      // Modules not from the Drupal.org repo are missing the 'project' key in *.info.
      $project = $shortname;
      $gotproject = FALSE;
    }
    else {
      $project = $info['project'];
      $gotproject = TRUE;
    }
    // Assumes that all projects have at least one module with the same name.
    if (!isset($projects[$project]) && ($project == $shortname)) {
      $module->longname = $info['name'];
      $module->description = $info['description'];
      $module->date = date('Y-m-d h:i', $info['mtime']);
      $module->shortname = $shortname;
      $module->longinfo = $info;
      $module->gotproject = $gotproject;
      $modarray[$info['name']] = $module;
    }
  }
  ksort($modarray, SORT_STRING | SORT_FLAG_CASE);
  $markup = '<h2>' . t('Summary') . '</h2><ol>';
  $create = $missing = $done = 0;
  foreach ($modarray as $module) {
    $status = _projectdocumentation_status($module->shortname);
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'projectdocumentation')
      ->propertyCondition('title', $module->longname);
    $result = $query->execute();
    if (empty($result['node'])) {
      if ($module->gotproject) {
        $markup .= '<li><strong>' . t('Project “@longname”</strong> - <code>!shortname</code> (!status) <strong>can be added to the project list.', array('@longname' => $module->longname, '!shortname' => $module->shortname, '!status' => $status)) . '</strong></li>';
	$form_state['pd']['cancreate'][] = $module;
	$create++;
      }
      else {
        $markup .= '<li>' . t('File <code>@shortname.info</code> for “@longname” is missing the “project” key.', array('@shortname' => $module->shortname, '@longname' => $module->longname)) . '</li>';
	$missing++;
      }
    }
    else {
      $markup .= '<li>' . t('Project “@longname” - <code>!shortname</code> (!status) is alredy documented.', array('@longname' => $module->longname, '!shortname' => $module->shortname, '!status' => $status)) . '</li>';
      $done++;
    }
  }
  $markup .= '</ol><p>';
  $markup .= t('!create project(s) can be documented. !missing project(s) missing “project” key. !done project(s) already documented.', array('!create' => $create, '!missing' => $missing, '!done' => $done));
  $markup .= '</p>';
  if ($missing) {
    $markup .= '<p>Projects where the “project” key is missing has not been packaged by Drupal.org. They are usually custom projects that has been added locally. You can add them to the project list manually or add the key manually.</p>';
  }
  $form['pd']['projectlist'] = array(
    '#markup' => $markup,
  );
  $form['datetimep'] = array(
      '#type' =>'checkbox',
      '#title'=>t("Fill in datetime field with today's date and time."),
      '#description'=>t("If not checked,  the node's date field will be empty."),
      '#default_value' =>TRUE,
  );

  if ($create) {
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Create documentation for !projects', array('!projects' => format_plural($create, 'one project', '@count projects'))),
    );
  }
  if (0 == $create) {
    $form['actions']['submit']['#disabled'] = TRUE;
  }
  if ($done) {
    $form['actions']['csv'] = array(
      '#type'   => 'button',
      '#value' => t('Export CSV'),
      '#executes_submit_callback' => TRUE,
      '#submit' => array('_projectdocumentation_form_csv'),
    );
  }
  $form['actions']['cancel'] = array(
    '#type'   => 'button',
    '#value' => t('Cancel'),
    '#executes_submit_callback' => TRUE,
    '#submit' => array('_projectdocumentation_form_cancel'),
  );
  return $form;
}

/**
 * Create documentation submit handler.
 */
function projectdocumentation_generate_submit($form, &$form_state) {
  foreach ($form_state['pd']['cancreate'] as $module) {
    _projectdocumentation_createnode($module, $form_state['values']['datetimep']);
  }
}

/**
 * Export CSV handler.
 */
function _projectdocumentation_form_csv(){
  drupal_goto('/admin/modules/projects');
}

/**
 * Cancel handler for form.
 */
function _projectdocumentation_form_cancel(){
  drupal_goto('/admin/config/system');
}

/**
 * Menu callback. Return render array for CSV export page.
 */
function projectdocumentation_export_csv() {
  $result = db_query("SELECT n.nid FROM {field_data_field_projdoc_shortname} sn
    INNER JOIN {node} n ON sn.entity_id = n.nid
    WHERE type = :type
    ORDER BY sn.field_projdoc_shortname_value ASC", array(':type' => 'projectdocumentation'));
  if ($result) {
    $ss = '';
    foreach ($result as $record) {
      $node = node_load($record->nid);
      $body = str_replace(';', ',' ,$node->body[LANGUAGE_NONE][0]['value']);
      $body = trim(htmlspecialchars(strip_tags($body, '<em><strong>')));
      $body = trim(preg_replace('/\s\s+/', ' ', $body));
      $ss .= $node->field_projdoc_shortname[LANGUAGE_NONE][0]['safe_value'] . ';' . $body . PHP_EOL;
    }
    $content['raw_markup'] = array(
      '#markup' => '<p>Copy &amp; paste the CSV below to export the projects.</p><pre>' . $ss . '</pre>',
    );

  }
  return $content;
}

/**
 * Callback to import CSV.
 */
function projectdocumentation_import_csv($form, &$form_state) {
  $form['textarea'] = array(
    '#title' => t('Import CSV'),
    '#type' => 'textarea',
    '#description' => 'Please copy &amp; paste the CSV in the area above. then press “Import”.',
    '#default_value' => '',
    '#rows' => 10,
    '#resizable' => TRUE,
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import'),
  );
  return $form;
}

/**
 * Import CSV submit handler.
 */
function projectdocumentation_import_csv_submit($form, &$form_state) {
  $nn = node_load(49);
  $lines = explode(PHP_EOL, $form_state['values']['textarea']);
  $descs = array();
  foreach ($lines as $line) {
    if (!empty($line)) {
      $els = explode(';', $line);
      $descs[$els[0]] = $els[1];
    }
  } 
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'projectdocumentation')
    ->propertyCondition('status', NODE_PUBLISHED)
    ->addMetaData('account', user_load(1));
  $result = $query->execute();
  if (isset($result['node'])) {
    $nids = array_keys($result['node']);
    $nodes = entity_load('node', $nids);
    foreach ($nodes as $node) {
      $shortname = $node->field_projdoc_shortname[LANGUAGE_NONE][0]['safe_value'];
      if (!empty($descs[$shortname])) {
	$node->body[LANGUAGE_NONE][0]['format'] = 'filtered_html';
	$node->body[LANGUAGE_NONE][0]['value'] = $descs[$shortname];
	$node->body[LANGUAGE_NONE][0]['safe_value'] = '<p>' . $descs[$shortname] . '</p>';
	node_save($node);
      }
    }
  } 
  drupal_set_message(t('CSV imported.'));
  drupal_goto('/admin/projectlist');
}

/**
 * Helper function to create projectdocumentation node.
 */
function _projectdocumentation_createnode($module, $dtp) {
global $user;
  $values = array(
    'type' => 'projectdocumentation',
    'uid' => $user->uid,
    'status' => 1,
    'comment' => 1,
    'promote' => 0,
  );
  $entity = entity_create('node', $values);
  $ewrapper = entity_metadata_wrapper('node', $entity);
  $ewrapper->title->set($module->longname);
  $ewrapper->body->set(array('value' => 'Explain why this project was installed.'));
  $ewrapper->field_projdoc_shortname->set($module->shortname);
  if ($dtp) {
    $ewrapper->field_projdoc_dateinst->set(REQUEST_TIME);
  }

  // Entity API cannot set some values so the old method must be used
  $entity->field_projdoc_projectpage[LANGUAGE_NONE][0]['url'] = 'https://www.drupal.org/project/' . $module->shortname;
  $entity->field_projdoc_projectpage[LANGUAGE_NONE][0]['attributes'] = array();

  $ewrapper->save();
  drupal_set_message(t('Adding <strong>!module</strong> to project list.', array('!module' => $module->longname)));
}
